'use strict';

const http = require('http');

http.IncomingMessage.prototype.body = function() {
  return new Promise(resolve => {
    this.on('data', c => resolve(c.toString()))
    .on('end', () => resolve('{}'))
  });
};

class Speedlane {
  constructor(port) {
    this._routes = {};
    this._middleware = [];
    
    http.METHODS.forEach(m => this._routes[m] = {});
    
    http.createServer((req, res) => {
      const cbs = this._middleware.concat(this._routes[req.method][req.url] || []);
      this._run(cbs, req, res);
    }).listen(port);
  }
  
  on(method, path, cb) {
    this._routes[method][path] = cb;
    return this;
  }
  
  use(cb) {
    this._middleware.unshift(cb);
    return this;
  }
  
  _run(cbs, req, res) {
    (async function next() {
      if(cbs.length) 
        await cbs.pop()(req, res, next);
    })();
  }
}

module.exports = { Speedlane, Request: http.IncomingMessage.prototype, Response: http.ServerResponse.prototype };